from webob import Request
import re

class DetectMobileBrowser(object):
    """Detect mobile browser

    Regexps reg_b and reg_v are taken from
    http://detectmobilebrowser.com/.  The regexps are in the public
    domain.
    
    Regexps updated: 30 June 2010.
    """
    reg_b = re.compile(r"android|avantgo|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\\/|plucker|pocket|psp|symbian|treo|up\\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino", re.I|re.M)
    
    reg_v = re.compile(r"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\\-(n|u)|c55\\/|capi|ccwa|cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560|gene|gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|hei\\-|hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |kwc\\-|kyo(c|k)|le(no|xi)|lg( g|\\/(k|l|u)|50|54|e\\-|e\\/|\\-[a-w])|libw|lynx|m1\\-w|m3ga|m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|po(ck|rt|se)|prox|psio|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|i\\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se(c(\\-|0|1)|47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|to(pl|sh)|ts(70|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|xda(\\-|2|g)|yas\\-|your|zeto|zte\\-", re.I|re.M)

    def __init__(self, application, config):
        super(DetectMobileBrowser, self).__init__()

        mobile_re_config = config.get('mobile.agents', False)
        if mobile_re_config:
            self.custom_mobile_re = re.compile(mobile_re_config, re.I|re.M)
            self._recognize_ua = self._recognize_ua_custom
        else:
            self.custom_mobile_re = None
            self._recognize_ua = self._recognize_ua_default
   
    def _recognize_ua_custom(self, user_agent):
        return self.custom_mobile_re.search(user_agent)

    def _recognize_ua_default(self, user_agent):
        return self.reg_b.search(user_agent) or self.reg_v.search(user_agent[:4])
 
    def perform_detection(self, request):
        user_agent = request.headers.get('User-Agent', None) 
        return user_agent and self._recognize_ua(user_agent)

    def __call__(self, request):
        return self.perform_detection(request)

class MobileMiddleware(object):
    def __init__(self, application, config,
                 mobile_browser_detector=DetectMobileBrowser):
        self.application = application
        
        if isinstance(mobile_browser_detector, type):
            self.detect_mobile = mobile_browser_detector(application, config)
        else:
            self.detect_mobile = mobile_browser_detector
            
    def __call__(self, environ, start_response):
        request = Request(environ)
        request.is_mobile = bool(self.detect_mobile(request))
        response = request.get_response(self.application)
        return response(environ, start_response)



from setuptools import setup, find_packages
import os

version = '0.4'

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(name='tgext.mobilemiddleware',
      version=version,
      description="Mobile Middleware utilities for Turbogears 2 detects mobile browsers",
      long_description=README,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Environment :: Web Environment",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: TurboGears"
        ],
      keywords='turbogears2.extension pylons',
      author='Alessandro Molina',
      author_email='alessandro.molina@axant.it',
      url='http://bitbucket.org/_amol_/tgext.mobilemiddleware',
      license='LGPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['tgext'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        "TurboGears2 >= 2.2.0"
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from tg import request, config
from tg.decorators import Decoration

class expose_mobile(object):
    def __init__(self, template, content_type='text/html'):
        if template in config.get('renderers', []):
            engine, template = template, ''
        elif ':' in template:
            engine, template = template.split(':', 1)
        elif template:
            engine = config.get('default_renderer')
        else:
            engine, template = None, None

        self.template = template
        self.engine = engine
        self.content_type = content_type

    def hook_func(self, *args, **kwargs):
        if request.is_mobile:
            try:
                override_mapping = request._override_mapping
            except:
                override_mapping = request._override_mapping = {}
            override_mapping[self.func] = {self.content_type:(self.engine, self.template, [], {})}

    def __call__(self, func):
        self.func = func
        deco = Decoration.get_decoration(func)
        deco.register_hook('before_render', self.hook_func)
        return func
